@@ -1,7 +1,9 @@
 import hashlib
+from functools import reduce
 
 from dezimal import Dezimal
 
+from .jsonutil import dumpjson, loadjson
 from .util import parse_name, empty
 
 
@@ -55,27 +57,40 @@ class Functions:
         return str(value).rstrip(chars)
 
     @staticmethod
-    def pad_left(value, char, total_length):
+    def strip(value, chars):
+        result = []
+        for char in value:
+            if char not in chars:
+                result.append(char)
+        return "".join(result)
+
+    @staticmethod
+    def _pad_prep(value, char, total_length):
         if not isinstance(value, str):
             value: str = str(value)
         if not isinstance(char, str):
             char: str = str(char)
         diff: int = int(total_length) - len(value)
+        return char, diff, value
+
+    @staticmethod
+    def pad_left(value, char, total_length):
+        char, diff, value = Functions._pad_prep(value, char, total_length)
         if diff <= 0:
             return value
         return char * diff + value
 
     @staticmethod
     def pad_right(value, char, total_length):
-        if not isinstance(value, str):
-            value: str = str(value)
-        if not isinstance(char, str):
-            char: str = str(char)
-        diff: int = int(total_length) - len(value)
+        char, diff, value = Functions._pad_prep(value, char, total_length)
         if diff <= 0:
             return value
         return value + char * diff
 
+    @staticmethod
+    def str_replace(needle, replacement, haystack):
+        return str(haystack).replace(needle, replacement)
+
     @staticmethod
     def explode(separator, value):
         return value.split(separator)
@@ -161,6 +176,14 @@ class Functions:
     def string(value):
         return str(value)
 
+    @staticmethod
+    def integer(value):
+        return int(value)
+
+    @staticmethod
+    def float(value):
+        return float(value)
+
     @staticmethod
     def boolean(value):
         return bool(value)
@@ -171,6 +194,14 @@ class Functions:
             return value
         return [value]
 
+    @staticmethod
+    def json_serialize(value):
+        return dumpjson(value)
+
+    @staticmethod
+    def json_read(value):
+        return loadjson(value)
+
     # aggregation
 
     @staticmethod
@@ -246,7 +277,13 @@ class Functions:
         return len(items)
 
     @staticmethod
-    def select(items, ix):
+    def select(items, ix, range_upper=None):
+        if isinstance(items, (list, tuple, str)):
+            ix = int(ix)
+            if range_upper is not None:
+                range_upper = int(range_upper)
+        if range_upper is not None:
+            return items[ix:range_upper]
         return items[ix]
 
     @staticmethod
@@ -272,9 +309,24 @@ class Functions:
         return reversed(items)
 
     @staticmethod
-    def sort(items):
+    def sort(*items):
         return sorted(items)
 
+    @staticmethod
+    def sort_by(key, items):
+        def select(item):
+            nonlocal key
+            if isinstance(key, str):
+                key = key.split('.')
+            for k in key:
+                try:
+                    item = item[k]
+                except KeyError:
+                    return None
+            return item
+
+        return sorted(items, key=select)
+
     @staticmethod
     def take(n, items):
         if not isinstance(items, list):
@@ -287,6 +339,34 @@ class Functions:
             items = str(items)
         return items[int(n):]
 
+    @staticmethod
+    def take_right(n, items):
+        if not isinstance(items, list):
+            items = str(items)
+        return items[-int(n):]
+
+    @staticmethod
+    def drop_right(n, items):
+        if not isinstance(items, list):
+            items = str(items)
+        return items[:-int(n)]
+
+    @staticmethod
+    def head(items):
+        return items[0]
+
+    @staticmethod
+    def last(items):
+        return items[-1]
+
+    @staticmethod
+    def tail(items):
+        return items[1:]
+
+    @staticmethod
+    def init(items):
+        return items[:-1]
+
     @staticmethod
     def concat(*items):
         if all(isinstance(item, list) for item in items):
@@ -294,8 +374,32 @@ class Functions:
             for ls in items:
                 for item in ls:
                     result.append(item)
+            return result
         return "".join([str(s) for s in items])
 
+    # lists only
+
+    @staticmethod
+    def flatten(items):
+        result = []
+        for item in items:
+            result.extend(item)
+        return result
+
+    @staticmethod
+    def deepflatten(*items):
+        result = []
+
+        def helper(xss):
+            if not isinstance(xss, (list, tuple)):
+                result.append(xss)
+                return
+            for xs in xss:
+                helper(xs)
+
+        helper(items)
+        return result
+
     # object creation utilities
 
     @staticmethod
@@ -313,3 +417,18 @@ class Functions:
             for key, value in item.items():
                 obj[key] = value
         return obj
+
+    @staticmethod
+    def deepmerge(*items):
+        def _merge(a, b):
+            for key in b:
+                if key in a:
+                    if isinstance(a[key], dict) and isinstance(b[key], dict):
+                        _merge(a[key], b[key])
+                    else:
+                        a[key] = b[key]
+                else:
+                    a[key] = b[key]
+            return a
+
+        return reduce(_merge, items)
